/*
 * 
 Developed on 25/2/20016
 by Abdel-Razzak Merheb
 Flying wing - Airplane Remote control using
 433 MHz RF Module Receiver Module
 (You need to choose between flying wing and airplane BEFORE start controlling)
 WITH SERVOs

 Base transmission code is developed by: 
 DroneBot Workshop 2018
 https://dronebotworkshop.com


VCC – Unlike the transmitter this needs to be 5 volts. We will use the 5 volt output from the Arduino.
GND – Again a ground that is connected to any Arduino ground pin.
DATA OUT – This needs to be connected to digital pin 11 on the Arduino.
*/

// ABD: Receive potentiometer data through 433 MHz 4/6/2021

// Include RadioHead Amplitude Shift Keying Library
#include <RH_ASK.h>
// Include dependant SPI Library 
#include <SPI.h> 

// Include the servo library that does NOT use Timer 1 (because RH_ASK uses timer 1 => conflict)
// #include <ServoTimer2.h> 

// Note: I (ABD) have changed the Timer used by RH_ASK and made it Timer2 (so I can use the regular servo.h library)
#include <Servo.h>


// Create the required servos
Servo Throttleservo; 
Servo Elevatorservo; 
Servo Rudderservo; 
Servo Aileronservo; 
Servo Gearservo; 


/*
// Create the required servos
ServoTimer2 Throttleservo; 
ServoTimer2 Elevatorservo; 
ServoTimer2 Rudderservo; 
ServoTimer2 Aileronservo; 
ServoTimer2 Gearservo; 
(
*/
// Create Amplitude Shift Keying Object
RH_ASK rf_driver;

// Pot variables as a structure
struct controlData{
int Throttle;     // The Throttle variable
int Rudder;     // The Rudder variable
int Elevator;     // The Elevator variable
int Aileron;     // The Aileron variable
int GearsDown;  // The gear down switch (initially fears are down -> active high)
int AirplaneFlyingWing;   // The variable: airplane OR flying wing?
int ElevTrim;  // The ELEVATOR trimmer value
int RudTrim;  // The RUDDER trimmer value
int AilTrim;  // The Aileron trimmer value
}myData;

// Define the control variables
int ThrottleValue;    // Define the Throttle variable
int RudderValue;    // Define the Throttle variable
int ElevatorValue;    // Define the Throttle variable
int AileronValue;    // Define the Throttle variable
int GearValue;    // Define the Gear variable
int AFSwitch;  // Airplane/flying wing switch: you need to choose BEFORE start controlling

void setup()
{
    // Initialize ASK Object
    rf_driver.init();
    // Setup Serial Monitor
    Serial.begin(9600);
    // Attach the servos to the relevant pin (PWM pins are 3, 5, 6, 9, 10, 11)
    Throttleservo.attach(3); 
    Elevatorservo.attach(5); 
    Rudderservo.attach(6); 
    Aileronservo.attach(9); 
    Gearservo.attach(10);     
}
 
void loop()
{ 
  //struct controlData buf[RH_ASK_MAX_MESSAGE_LEN];
// Receive potentiometer data from the 433 MhZ
uint8_t buf[RH_ASK_MAX_MESSAGE_LEN];
// uint8_t buflen = sizeof(buf);
uint8_t buflen = sizeof(myData);

  //if (rf_driver.recv(buf,&buflen))        // Non-blocking
  if (rf_driver.recv((uint8_t*) &myData,&buflen))        // Non-blocking
    {

      ThrottleValue = myData.Throttle;    // Read the Throttle variable
      RudderValue = myData.Rudder;    // Read the Throttle variable
      ElevatorValue = myData.Elevator;    // Read the Throttle variable
      AileronValue = myData.Aileron;    // Read the Throttle variable
      GearValue = myData.GearsDown;    // Read the Gear variable
      AFSwitch = myData.AirplaneFlyingWing;   // Read the type of the airplane (flying wing OR airplane?)
             
    }    

      // memcpy(&myData, buf, sizeof(buflen));      

      // myservo.write(myData.angle); // use the data directly to control servo
      
      // Potentiometer values received
      Serial.print("Throttle value: ");
      Serial.println(ThrottleValue); 
      Serial.print("Rudder value: ");
      Serial.println(RudderValue); 
      Serial.print("Elevator value: ");
      Serial.println(ElevatorValue); 
      Serial.print("Aileron value: ");
      Serial.println(AileronValue); 
      Serial.print("Gears are: ");
      Serial.println(GearValue);
      Serial.print("Airplane (1) OR flying wing (0)?");
      Serial.println(AFSwitch);
      Serial.print("Elevator trim value: ");
      Serial.println(myData.ElevTrim);    
      Serial.print("Rudder trim value: ");
      Serial.println(myData.RudTrim);  
      Serial.print("Aileron trim value: ");
      Serial.println(myData.AilTrim);        
      // Serial.println((char*myData.Throttle);  

      delay(100);

      // If Airplane is chosen to be controlled
      if(AFSwitch == 1)
      {
        Serial.print("I am a happy airplane");
        // Control the servomotors according to the received values (for ServoTimer2, instead of 0-90-180 angles we use 1000-2000 angles)
        Throttleservo.write(map(myData.Throttle,0,100,1000,2000));  // Realize the value received from the transmitter
        Elevatorservo.write(map(myData.Elevator,0,100,1000,2000) + myData.ElevTrim);  // Realize the value received from the transmitter + the trimmed value
        Rudderservo.write(map(myData.Rudder,0,100,1000,2000) + myData.RudTrim);  // Realize the value received from the transmitter + the trimmed value
        Aileronservo.write(map(myData.Aileron,0,100,1000,2000) + myData.AilTrim);  // Realize the value received from the transmitter + the trimmed value
        Gearservo.write(map(myData.GearsDown,0,1,1000,2000));  // Realize the value received from the transmitter
/*
        // Control the servomotors according to the received values (for ServoTimer2, instead of 0-90-180 angles we use 1000-2000 angles)
        Throttleservo.write(map(myData.Throttle,0,100,0,180));  // Realize the value received from the transmitter
        Elevatorservo.write(map(myData.Elevator,0,100,0,180) + myData.ElevTrim);  // Realize the value received from the transmitter + the trimmed value
        Rudderservo.write(map(myData.Rudder,0,100,0,180) + myData.RudTrim);  // Realize the value received from the transmitter + the trimmed value
        Aileronservo.write(map(myData.Aileron,0,100,0,180) + myData.AilTrim);  // Realize the value received from the transmitter + the trimmed value
        Gearservo.write(map(myData.GearsDown,0,1,0,180));  // Realize the value received from the transmitter     
*/
      }

      // If Flying wing is chosen to be controlled
      if(AFSwitch == 0)
      {
        Serial.print("I am a happy flyin wing");
        // Control the servomotors according to the received values (for ServoTimer2, instead of 0-90-180 angles we use 1000-2000 angles)
        Throttleservo.write(map(myData.Throttle,0,100,1000,2000));  // Realize the value received from the transmitter
        Gearservo.write(map(myData.GearsDown,0,1,1000,2000));  // Realize the value received from the transmitter
        // Elevator servo is for RIGHT Elevon || Aileron servo is for LEFT Elevon
        // If Elevator is changing => ignore ailerons, apply altitude increase only
        // If no Elevator => apply aileron
        if(myData.Elevator < 55 && myData.Elevator > 45) // No elevator
        { 
          // BOTH are ailerons
          Elevatorservo.write(map(myData.Aileron,0,100,1000,2000) + myData.AilTrim);  // Realize the value received from the transmitter + the trimmed value
          Aileronservo.write(map(100 - myData.Aileron,0,100,1000,2000) + myData.AilTrim);  // Realize the COMPLEMENT of the value received from the transmitter + the trimmed value
        }

        if(myData.Elevator < 45 || myData.Elevator > 55) // There is elevator => ignore the ailerons
        { 
          // BOTH are elevators
          Elevatorservo.write(map(myData.Elevator,0,100,1000,2000) + myData.ElevTrim);  // Realize the value received from the transmitter for the elevator + the trimmed value
          Aileronservo.write(map(myData.Elevator,0,100,1000,2000) + myData.ElevTrim);  // Realize the value received from the transmitter for the elevator + the trimmed value
        }
               
/*
        // Control the servomotors according to the received values (for ServoTimer2, instead of 0-90-180 angles we use 1000-2000 angles)
        Throttleservo.write(map(myData.Throttle,0,100,0,180));  // Realize the value received from the transmitter
        Elevatorservo.write(map(myData.Elevator,0,100,0,180) + myData.ElevTrim);  // Realize the value received from the transmitter + the trimmed value
        Rudderservo.write(map(myData.Rudder,0,100,0,180) + myData.RudTrim);  // Realize the value received from the transmitter + the trimmed value
        Aileronservo.write(map(myData.Aileron,0,100,0,180) + myData.AilTrim);  // Realize the value received from the transmitter + the trimmed value
        Gearservo.write(map(myData.GearsDown,0,1,0,180));  // Realize the value received from the transmitter     
*/
      }

      delay(1000);       
}
